#!/usr/bin/env perl

# NAME:
#	pcmeter.pl - generate a precent complete indicator.
#
# SYNOPSIS:
#	$bar = &pcent_meter($n,$total,$width,$prefix,$head,$body)
#	pcmeter.pl "n" "total" ["width"] ["prefix"]
#	pcmeter.pl -t "total" [-N][-w "width"] [-n "n"] [-m "max"] [-p "prefix"] [-T "timeout"]
#
# DESCRIPTION:
#	This routine will generate a string suitable for displaying
#	the percentage complete.  For example:
#.nf
#
#	printf "\\r%s\\n", &pcent_meter(6,11,50,"Transfer");
#
#	produces:
#	
#	Transfer |###############              |  54.55%
#.fi
#
#	It can also be run as a command line tool.
#
#	If "head" is given as '/' or if the percent complete is more
#	than 100, we will ``twiddle'' the end of the bar.  If not
#	specified it defaults to '#'.
#
#	If "body" is specified it is used for the progress bar other
#	than the last character.  It defaults to '#'.
#	
#	If '-t' is given, it loops incrementing "n" for each line read
#	from stdin and printing the "bar" when it changes.  If '-N' is
#	also given a newline is printed after the "bar" rather than a
#	carriage return - this is only useful if other output would
#	obscure the progress meter.
#
#	If '-t' isn't a numeric expression, it should describe how to
#	obtain a number from the input eg. "some pattern n=(\d+)"
#	
#	Without the '-t' option it just prints a "bar" representing
#	the args given to it.
#
#	The '-t' option is handy for tracking the progress of
#	something like install_patches(8) as in:
#.nf
#
#	tail -f /tmp/config_1234/install_patches.1234.checked | \\
#	pcmeter.pl -t `wc -l < /tmp/config_1234/install_patches.1234.list`
#.fi
#
#	Which is what install_patches(8) does if "PCMETER_TTY" is set.
#	
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>

# RCSid:
#	$Id: pcmeter.pl,v 1.12 2013/06/05 20:18:26 sjg Exp $
#
#	@(#)Copyright (c) 1998-2013 Simon J. Gerraty. All rights reserved.
#	
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to use, modify and distribute this source code 
#	is granted subject to the following conditions.
#	1/ that the above copyright notice and this notice 
#	are preserved in all copies and that due credit be given 
#	to the author.	
#	2/ that any changes to this code are clearly commented 
#	as such so that the author does not get blamed for bugs 
#	other than his own.
#	3/ If there is an accompanying License file its provisions
#	must also be met.
#	
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

package pcmeter;

$Twiddle_ix = 0;
@Twiddles = ('/','-','\\','|');
$Lbar = '-';
@BChars = ('#',  # head of the bar '/' to twidle
	   '#'); # body of the bar

sub main'twiddle {
  local($x) = @_;

  if ($x ne '') {
    $Twiddle_ix = $x;
  } else {
    ++$Twiddle_ix;
  }
  $Twiddle_ix = 0 if ($Twiddle_ix > $#Twiddles);
    
  $Twiddles[$Twiddle_ix];
}

sub main'pcent_meter {
  local($n,$total,$width,$prefix,@bc) = @_;
  local($pcent) = $n / $total;
  local($bpcent) = $pcent;
  local($bar) = '';
  local($twdl);
  local($i);
  
  $width = 80 if ($width < 10);
  $prefix = '' if ($prefix eq '-');
  if (($i = scalar(@bc)) == 0) {
    @bc = @BChars;
  } elsif ($i == 1) {
    if ($bc[0] eq '/') {
      push(@bc, '#');
    } else {
      push(@bc, $bc[0]);
    }
  }
  if ($prefix ne '') {
    $prefix .= ' ';
    $width -= length($prefix);
  }
  $width -= 12;
  $bpcent = 1 if ($bpcent > 1);

  for ($i = 1; $i < int($width * $bpcent); ++$i) {
    $bar .= $bc[1];
  }
  $twdl = ($Lbar eq $bar);
  $Lbar = $bar;
  if ($i <= $width) {
    if ($bc[0] ne '/' && $pcent < 1) {
      $bar .= $bc[0];
    } elsif ($twdl || $pcent > 1) {
      $bar .= &main'twiddle(); # 'emacs
    } else {
      $bar .= $bc[1];
    }
  }
  sprintf "%s|%-*s| %6.2f%%", $prefix, $width, $bar, 100 * $pcent;
}

if ($0 =~ m/pcmeter/) {
  require 'getopts.pl';
  $opt_n = 0;
  $opt_T = 0;
  
  &Getopts('T:t:w:p:m:Nn:e:E:');

  if ($opt_t eq '') {
    printf  "\r%s\n", &main'pcent_meter(@ARGV); # 'emacs
    exit 0;
  }
  $opt_N = "\n" if ($opt_N ne '');
    
  if ($opt_t =~ /^\d/) {
    $opt_m = int($opt_t * 1.5) if ($opt_m eq '');
  }
  $lbar = '';
  $ltime = 0;
  $last_n = -1;

  alarm($opt_T);
  while (<STDIN>) {
    alarm($opt_T);
    if ($opt_E ne '' && m/$opt_E/) {
      ++$opt_n;
      $opt_m = $opt_n;		# so we bail
    } elsif ($opt_e eq '' || m/$opt_e/) {
      ++$opt_n;
    }
    if ($opt_t !~ /^\d/) {
      if (m/$opt_t/) {
	$opt_t = $1;
	$opt_m = int($opt_t * 1.5) if ($opt_m eq '');
      }
      next;
    }
    $now = time;
    next if ($now == $ltime);
    $ltime = $now;
    $bar = &main'pcent_meter($opt_n,$opt_t,$opt_w,$opt_p,
			     ($last_n == $opt_n) ? '/' : '#'); # 'emacs
    $last_n = $opt_n;
    if ($lbar ne $bar) {
      printf STDERR "\r%s$opt_N", $bar;
      $lbar = $bar;
    }
    last if ($opt_m > 0 && $opt_n >= $opt_m);
  }
  printf STDERR "\r%s\n", &main'pcent_meter($opt_n,$opt_t,$opt_w,$opt_p); # 'emacs
}
1;

  
